@@ -1,6 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2017 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -35,58 +36,7 @@ SearchSuggester::SearchSuggester(const QString &searchEngine, QObject *parent) :
 {
 }
 
-void SearchSuggester::setSearchEngine(const QString &searchEngine)
-{
-	const QString query(m_query);
-
-	m_searchEngine = searchEngine;
-	m_query = QString();
-
-	setQuery(query);
-}
-
-void SearchSuggester::setQuery(const QString &query)
-{
-	if (query != m_query)
-	{
-		m_query = query;
-
-		if (m_networkReply)
-		{
-			m_networkReply->abort();
-			m_networkReply->deleteLater();
-			m_networkReply = nullptr;
-		}
-
-		const SearchEnginesManager::SearchEngineDefinition searchEngine(SearchEnginesManager::getSearchEngine(m_searchEngine));
-
-		if (searchEngine.identifier.isEmpty() || searchEngine.suggestionsUrl.url.isEmpty())
-		{
-			return;
-		}
-
-		QNetworkRequest request;
-		request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
-
-		QNetworkAccessManager::Operation method;
-		QByteArray body;
-
-		SearchEnginesManager::setupQuery(query, searchEngine.suggestionsUrl, &request, &method, &body);
-
-		if (method == QNetworkAccessManager::PostOperation)
-		{
-			m_networkReply = NetworkManagerFactory::getNetworkManager()->post(request, body);
-		}
-		else
-		{
-			m_networkReply = NetworkManagerFactory::getNetworkManager()->get(request);
-		}
-
-		connect(m_networkReply, SIGNAL(finished()), this, SLOT(replyFinished()));
-	}
-}
-
-void SearchSuggester::replyFinished()
+void SearchSuggester::handleReplyFinished()
 {
 	if (!m_networkReply)
 	{
@@ -114,7 +64,8 @@ void SearchSuggester::replyFinished()
 		const QJsonArray completionsArray(document.array().at(1).toArray());
 		const QJsonArray descriptionsArray(document.array().at(2).toArray());
 		const QJsonArray urlsArray(document.array().at(3).toArray());
-		QList<SearchSuggestion> suggestions;
+
+		m_suggestions.reserve(completionsArray.count());
 
 		for (int i = 0; i < completionsArray.count(); ++i)
 		{
@@ -123,28 +74,92 @@ void SearchSuggester::replyFinished()
 			suggestion.description = descriptionsArray.at(i).toString();
 			suggestion.url = urlsArray.at(i).toString();
 
+			m_suggestions.append(suggestion);
+
 			if (m_model)
 			{
 				m_model->appendRow(new QStandardItem(suggestion.completion));
 			}
-
-			suggestions.append(suggestion);
 		}
 
-		emit suggestionsChanged(suggestions);
+		emit suggestionsChanged(m_suggestions);
 	}
 
 	m_networkReply = nullptr;
 }
 
+void SearchSuggester::setSearchEngine(const QString &searchEngine)
+{
+	const QString query(m_query);
+
+	m_searchEngine = searchEngine;
+	m_query.clear();
+
+	setQuery(query);
+}
+
+void SearchSuggester::setQuery(const QString &query)
+{
+	if (query == m_query)
+	{
+		return;
+	}
+
+	m_query = query;
+	m_suggestions.clear();
+
+	if (m_networkReply)
+	{
+		m_networkReply->abort();
+		m_networkReply->deleteLater();
+		m_networkReply = nullptr;
+	}
+
+	const SearchEnginesManager::SearchEngineDefinition searchEngine(SearchEnginesManager::getSearchEngine(m_searchEngine));
+
+	if (!searchEngine.isValid() || searchEngine.suggestionsUrl.url.isEmpty())
+	{
+		return;
+	}
+
+	QNetworkRequest request;
+	request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
+
+	QNetworkAccessManager::Operation method;
+	QByteArray body;
+
+	SearchEnginesManager::setupQuery(query, searchEngine.suggestionsUrl, &request, &method, &body);
+
+	if (method == QNetworkAccessManager::PostOperation)
+	{
+		m_networkReply = NetworkManagerFactory::getNetworkManager()->post(request, body);
+	}
+	else
+	{
+		m_networkReply = NetworkManagerFactory::getNetworkManager()->get(request);
+	}
+
+	connect(m_networkReply, &QNetworkReply::finished, this, &SearchSuggester::handleReplyFinished);
+}
+
 QStandardItemModel* SearchSuggester::getModel()
 {
 	if (!m_model)
 	{
 		m_model = new QStandardItemModel(this);
+
+		for (int i = 0; i < m_suggestions.count(); ++i)
+		{
+			m_model->appendRow(new QStandardItem(m_suggestions.at(i).completion));
+		}
 	}
 
 	return m_model;
 }
 
+QVector<SearchSuggester::SearchSuggestion> SearchSuggester::getSuggestions() const
+{
+	return m_suggestions;
+}
+
 }
