@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2017 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2012 by Lindsay Mathieson <lindsay dot mathieson at gmail dot com>
 * Copyright (C) 2012 by Andrea Diamantini <adjam7 at gmail dot com>
 *
@@ -11,11 +11,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -27,20 +27,13 @@
 namespace Otter
 {
 
-QtWebKitSpellChecker::QtWebKitSpellChecker() : QWebSpellChecker(),
-	m_speller(nullptr)
+Sonnet::Speller* QtWebKitSpellChecker::m_speller(nullptr);
+
+QtWebKitSpellChecker::QtWebKitSpellChecker() : QWebSpellChecker()
 {
 	setDictionary(QtWebKitWebBackend::getActiveDictionary());
 
-	connect(QtWebKitWebBackend::getInstance(), SIGNAL(activeDictionaryChanged(QString)), this, SLOT(setDictionary(QString)));
-}
-
-QtWebKitSpellChecker::~QtWebKitSpellChecker()
-{
-	if (m_speller)
-	{
-		delete m_speller;
-	}
+	connect(QtWebKitWebBackend::getInstance(), &QtWebKitWebBackend::activeDictionaryChanged, this, &QtWebKitSpellChecker::setDictionary);
 }
 
 void QtWebKitSpellChecker::toggleContinousSpellChecking()
@@ -158,7 +151,22 @@ QString QtWebKitSpellChecker::autoCorrectSuggestionForMisspelledWord(const QStri
 {
 	Q_UNUSED(word)
 
-	return QString();
+	return {};
+}
+
+QStringList QtWebKitSpellChecker::getSuggestions(const QString &word)
+{
+	if (!m_speller)
+	{
+		m_speller = new Sonnet::Speller(QtWebKitWebBackend::getActiveDictionary());
+	}
+
+	if (m_speller->isCorrect(word))
+	{
+		return {};
+	}
+
+	return m_speller->suggest(word);
 }
 
 bool QtWebKitSpellChecker::isContinousSpellCheckingEnabled() const
