@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2010 David Sansome <me@davidsansome.com>
 * Copyright (C) 2015 Piotr Wójcik <chocimier@tlen.pl>
 *
@@ -11,27 +11,32 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "FreeDesktopOrgPlatformIntegration.h"
+#include "FreeDesktopOrgPlatformStyle.h"
 #include "../../../core/Application.h"
 #include "../../../core/NotificationsManager.h"
 #include "../../../core/SettingsManager.h"
 #include "../../../core/TransfersManager.h"
+#include "../../../core/Utils.h"
 #include "../../../../3rdparty/libmimeapps/DesktopEntry.h"
 #include "../../../../3rdparty/libmimeapps/Index.h"
 
 #include <QtConcurrent/QtConcurrent>
+#ifdef OTTER_ENABLE_DBUS
 #include <QtDBus/QtDBus>
 #include <QtDBus/QDBusReply>
+#endif
 #include <QtGui/QDesktopServices>
 #include <QtGui/QIcon>
+#ifdef OTTER_ENABLE_DBUS
 #include <QtGui/QRgb>
 
 QDBusArgument& operator<<(QDBusArgument &argument, const QImage &image)
@@ -92,37 +97,52 @@ const QDBusArgument& operator>>(const QDBusArgument &argument, QImage &image)
 
 	return argument;
 }
+#endif
 
 namespace Otter
 {
 
-FreeDesktopOrgPlatformIntegration::FreeDesktopOrgPlatformIntegration(Application *parent) : PlatformIntegration(parent),
-	m_notificationsInterface(new QDBusInterface(QLatin1String("org.freedesktop.Notifications"), QLatin1String("/org/freedesktop/Notifications"), QLatin1String("org.freedesktop.Notifications"), QDBusConnection::sessionBus(), this))
+FreeDesktopOrgPlatformIntegration::FreeDesktopOrgPlatformIntegration(QObject *parent) : PlatformIntegration(parent)
+#ifdef OTTER_ENABLE_DBUS
+	, m_notificationsInterface(new QDBusInterface(QLatin1String("org.freedesktop.Notifications"), QLatin1String("/org/freedesktop/Notifications"), QLatin1String("org.freedesktop.Notifications"), QDBusConnection::sessionBus(), this))
+#endif
 {
 #if QT_VERSION >= 0x050700
 	QGuiApplication::setDesktopFileName(QLatin1String("otter-browser.desktop"));
 #endif
 
+#ifdef OTTER_ENABLE_DBUS
 	qDBusRegisterMetaType<QImage>();
 
 	m_notificationsInterface->connection().connect(m_notificationsInterface->service(), m_notificationsInterface->path(), m_notificationsInterface->interface(), QLatin1String("NotificationClosed"), this, SLOT(handleNotificationIgnored(quint32,quint32)));
 	m_notificationsInterface->connection().connect(m_notificationsInterface->service(), m_notificationsInterface->path(), m_notificationsInterface->interface(), QLatin1String("ActionInvoked"), this, SLOT(handleNotificationClicked(quint32,QString)));
 
 	updateTransfersProgress();
+#endif
 
-	QTimer::singleShot(250, this, SLOT(createApplicationsCacheThread()));
+	QTimer::singleShot(250, this, [&]()
+	{
+		QtConcurrent::run([&]()
+		{
+			getApplicationsForMimeType(QMimeDatabase().mimeTypeForName(QLatin1String("text/html")));
+		});
+	});
 
-	connect(TransfersManager::getInstance(), SIGNAL(transferChanged(Transfer*)), this, SLOT(updateTransfersProgress()));
-	connect(TransfersManager::getInstance(), SIGNAL(transferStarted(Transfer*)), this, SLOT(updateTransfersProgress()));
-	connect(TransfersManager::getInstance(), SIGNAL(transferFinished(Transfer*)), this, SLOT(updateTransfersProgress()));
-	connect(TransfersManager::getInstance(), SIGNAL(transferRemoved(Transfer*)), this, SLOT(updateTransfersProgress()));
-	connect(TransfersManager::getInstance(), SIGNAL(transferStopped(Transfer*)), this, SLOT(updateTransfersProgress()));
+#ifdef OTTER_ENABLE_DBUS
+	connect(TransfersManager::getInstance(), &TransfersManager::transferChanged, this, &FreeDesktopOrgPlatformIntegration::updateTransfersProgress);
+	connect(TransfersManager::getInstance(), &TransfersManager::transferStarted, this, &FreeDesktopOrgPlatformIntegration::updateTransfersProgress);
+	connect(TransfersManager::getInstance(), &TransfersManager::transferFinished, this, &FreeDesktopOrgPlatformIntegration::updateTransfersProgress);
+	connect(TransfersManager::getInstance(), &TransfersManager::transferRemoved, this, &FreeDesktopOrgPlatformIntegration::updateTransfersProgress);
+	connect(TransfersManager::getInstance(), &TransfersManager::transferStopped, this, &FreeDesktopOrgPlatformIntegration::updateTransfersProgress);
+#endif
 }
 
+#ifdef OTTER_ENABLE_DBUS
 FreeDesktopOrgPlatformIntegration::~FreeDesktopOrgPlatformIntegration()
 {
-	updateTransfersProgress(true);
+	setTransfersProgress(0, 0, 0);
 }
+#endif
 
 void FreeDesktopOrgPlatformIntegration::runApplication(const QString &command, const QUrl &url) const
 {
@@ -134,7 +154,7 @@ void FreeDesktopOrgPlatformIntegration::runApplication(const QString &command, c
 	std::vector<std::string> fileNames;
 	fileNames.push_back((url.isLocalFile() ? QDir::toNativeSeparators(url.toLocalFile()) : url.url()).toStdString());
 
-	std::vector<std::string> parsed(LibMimeApps::DesktopEntry::parseExec(command.toStdString(), fileNames));
+	const std::vector<std::string> parsed(LibMimeApps::DesktopEntry::parseExec(command.toStdString(), fileNames, LibMimeApps::DesktopEntry::ParseOptions::NecessarilyUseUrl));
 
 	if (parsed.size() < 1)
 	{
@@ -142,6 +162,7 @@ void FreeDesktopOrgPlatformIntegration::runApplication(const QString &command, c
 	}
 
 	QStringList arguments;
+	arguments.reserve(static_cast<int>(parsed.size()));
 
 	for (std::vector<std::string>::size_type i = 1; i < parsed.size(); ++i)
 	{
@@ -151,16 +172,7 @@ void FreeDesktopOrgPlatformIntegration::runApplication(const QString &command, c
 	QProcess::startDetached(QString::fromStdString(parsed.at(0)), arguments);
 }
 
-void FreeDesktopOrgPlatformIntegration::createApplicationsCache()
-{
-	m_applicationsCache[QLatin1String("text/html")] = getApplicationsForMimeType(QMimeDatabase().mimeTypeForName(QLatin1String("text/html")));
-}
-
-void FreeDesktopOrgPlatformIntegration::createApplicationsCacheThread()
-{
-	QtConcurrent::run(this, &FreeDesktopOrgPlatformIntegration::createApplicationsCache);
-}
-
+#ifdef OTTER_ENABLE_DBUS
 void FreeDesktopOrgPlatformIntegration::handleNotificationCallFinished(QDBusPendingCallWatcher *watcher)
 {
 	Notification *notification(m_notificationWatchers.value(watcher, nullptr));
@@ -183,7 +195,7 @@ void FreeDesktopOrgPlatformIntegration::handleNotificationIgnored(quint32 identi
 
 	if (notification)
 	{
-		notification->markIgnored();
+		notification->markAsIgnored();
 
 		m_notifications.remove(identifier);
 	}
@@ -197,7 +209,7 @@ void FreeDesktopOrgPlatformIntegration::handleNotificationClicked(quint32 identi
 
 	if (notification)
 	{
-		notification->markClicked();
+		notification->markAsClicked();
 
 		m_notifications.remove(identifier);
 	}
@@ -223,10 +235,7 @@ void FreeDesktopOrgPlatformIntegration::showNotification(Notification *notificat
 			break;
 	}
 
-	const int visibilityDuration(SettingsManager::getValue(SettingsManager::Interface_NotificationVisibilityDurationOption).toInt());
-	QVariantMap hints;
-	hints[QLatin1String("image_data")] = Application::windowIcon().pixmap(128, 128).toImage();
-
+	const int visibilityDuration(SettingsManager::getOption(SettingsManager::Interface_NotificationVisibilityDurationOption).toInt());
 	QVariantList arguments;
 	arguments << Application::applicationName();
 	arguments << uint(0);
@@ -234,65 +243,80 @@ void FreeDesktopOrgPlatformIntegration::showNotification(Notification *notificat
 	arguments << tr("Notification");
 	arguments << notification->getMessage();
 	arguments << QStringList();
-	arguments << hints;
+	arguments << QVariantMap({{QLatin1String("image_data"), Application::windowIcon().pixmap(128, 128).toImage()}});
 	arguments << ((visibilityDuration < 0) ? -1 : (visibilityDuration * 1000));
 
 	QDBusPendingCallWatcher *watcher(new QDBusPendingCallWatcher(m_notificationsInterface->asyncCallWithArgumentList(QLatin1String("Notify"), arguments), this));
 
 	m_notificationWatchers[watcher] = notification;
 
-	connect(watcher, SIGNAL(finished(QDBusPendingCallWatcher*)), this, SLOT(handleNotificationCallFinished(QDBusPendingCallWatcher*)));
+	connect(watcher, &QDBusPendingCallWatcher::finished, this, &FreeDesktopOrgPlatformIntegration::handleNotificationCallFinished);
 }
 
-void FreeDesktopOrgPlatformIntegration::updateTransfersProgress(bool clear)
+void FreeDesktopOrgPlatformIntegration::updateTransfersProgress()
 {
 	qint64 bytesTotal(0);
 	qint64 bytesReceived(0);
 	qint64 transferAmount(0);
 
-	if (!clear)
+	if (TransfersManager::hasRunningTransfers())
 	{
-		const QList<Transfer*> transfers(TransfersManager::getInstance()->getTransfers());
+		const QVector<Transfer*> transfers(TransfersManager::getInstance()->getTransfers());
 
 		for (int i = 0; i < transfers.count(); ++i)
 		{
-			if (transfers[i]->getState() == Transfer::RunningState && transfers[i]->getBytesTotal() > 0)
+			const Transfer *transfer(transfers.at(i));
+
+			if (transfer->getState() == Transfer::RunningState && transfer->getBytesTotal() > 0)
 			{
 				++transferAmount;
 
-				bytesTotal += transfers[i]->getBytesTotal();
-				bytesReceived += transfers[i]->getBytesReceived();
+				bytesTotal += transfer->getBytesTotal();
+				bytesReceived += transfer->getBytesReceived();
 			}
 		}
 	}
 
+	setTransfersProgress(bytesTotal, bytesReceived, transferAmount);
+}
+
+void FreeDesktopOrgPlatformIntegration::setTransfersProgress(qint64 bytesTotal, qint64 bytesReceived, qint64 transferAmount)
+{
 	const bool hasActiveTransfers(transferAmount > 0);
 	QVariantMap properties;
 	properties[QLatin1String("count")] = transferAmount;
 	properties[QLatin1String("count-visible")] = hasActiveTransfers;
-	properties[QLatin1String("progress")] = ((bytesReceived > 0) ? (static_cast<qreal>(bytesReceived) / bytesTotal) : 0.0);
+	properties[QLatin1String("progress")] = ((bytesReceived > 0) ? Utils::calculatePercent(bytesReceived, bytesTotal, 1) : 0.0);
 	properties[QLatin1String("progress-visible")] = hasActiveTransfers;
 
-	QVariantList arguments;
-	arguments << QLatin1String("application://otter-browser.desktop");
-	arguments << properties;
-
 	QDBusMessage message(QDBusMessage::createSignal(QLatin1String("/com/canonical/unity/launcherentry/9ddcf02c30a33cd63e9762f06957263f"), QLatin1String("com.canonical.Unity.LauncherEntry"), QLatin1String("Update")));
-	message.setArguments(arguments);
+	message.setArguments({QLatin1String("application://otter-browser.desktop"), properties});
 
 	QDBusConnection::sessionBus().send(message);
 }
+#endif
 
-QList<ApplicationInformation> FreeDesktopOrgPlatformIntegration::getApplicationsForMimeType(const QMimeType &mimeType)
+Style* FreeDesktopOrgPlatformIntegration::createStyle(const QString &name) const
+{
+	if (name.isEmpty() || name.toLower().startsWith(QLatin1String("gtk")))
+	{
+		return new FreeDesktopOrgPlatformStyle(name);
+	}
+
+	return nullptr;
+}
+
+QVector<ApplicationInformation> FreeDesktopOrgPlatformIntegration::getApplicationsForMimeType(const QMimeType &mimeType)
 {
 	if (m_applicationsCache.contains(mimeType.name()))
 	{
 		return m_applicationsCache[mimeType.name()];
 	}
 
-	LibMimeApps::Index index(QLocale().bcp47Name().toStdString());
-	std::vector<LibMimeApps::DesktopEntry> applications(index.appsForMime(mimeType.name().toStdString()));
-	QList<ApplicationInformation> result;
+	const LibMimeApps::Index index(QLocale().bcp47Name().toStdString());
+	const std::vector<LibMimeApps::DesktopEntry> applications(index.appsForMime(mimeType.name().toStdString()));
+	QVector<ApplicationInformation> result;
+	result.reserve(static_cast<int>(applications.size()));
 
 	for (std::vector<LibMimeApps::DesktopEntry>::size_type i = 0; i < applications.size(); ++i)
 	{
@@ -309,9 +333,11 @@ QList<ApplicationInformation> FreeDesktopOrgPlatformIntegration::getApplications
 	return result;
 }
 
+#ifdef OTTER_ENABLE_DBUS
 bool FreeDesktopOrgPlatformIntegration::canShowNotifications() const
 {
 	return m_notificationsInterface->isValid();
 }
+#endif
 
 }
