@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2016 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2016 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
@@ -10,17 +10,17 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "AddonsContentsWidget.h"
-#include "../../../core/ActionsManager.h"
 #include "../../../core/SessionsManager.h"
+#include "../../../core/JsonSettings.h"
 #include "../../../core/ThemesManager.h"
 #include "../../../core/UserScript.h"
 #include "../../../core/Utils.h"
@@ -28,7 +28,6 @@
 #include "ui_AddonsContentsWidget.h"
 
 #include <QtCore/QFile>
-#include <QtCore/QJsonDocument>
 #include <QtCore/QJsonObject>
 #include <QtCore/QStandardPaths>
 #include <QtCore/QTimer>
@@ -41,21 +40,21 @@
 namespace Otter
 {
 
-AddonsContentsWidget::AddonsContentsWidget(Window *window) : ContentsWidget(window),
+AddonsContentsWidget::AddonsContentsWidget(const QVariantMap &parameters, Window *window, QWidget *parent) : ContentsWidget(parameters, window, parent),
 	m_model(new QStandardItemModel(this)),
 	m_isLoading(true),
 	m_ui(new Ui::AddonsContentsWidget)
 {
 	m_ui->setupUi(this);
-	m_ui->addonsViewWidget->setViewMode(ItemViewWidget::TreeViewMode);
+	m_ui->filterLineEditWidget->setClearOnEscape(true);
+	m_ui->addonsViewWidget->setViewMode(ItemViewWidget::TreeView);
 	m_ui->addonsViewWidget->installEventFilter(this);
-	m_ui->filterLineEdit->installEventFilter(this);
 
-	QTimer::singleShot(100, this, SLOT(populateAddons()));
+	QTimer::singleShot(100, this, &AddonsContentsWidget::populateAddons);
 
-	connect(m_ui->filterLineEdit, SIGNAL(textChanged(QString)), this, SLOT(filterAddons(QString)));
-	connect(m_ui->addonsViewWidget, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(showContextMenu(QPoint)));
-	connect(m_ui->addonsViewWidget, SIGNAL(clicked(QModelIndex)), this, SLOT(save()));
+	connect(m_ui->filterLineEditWidget, &LineEditWidget::textChanged, m_ui->addonsViewWidget, &ItemViewWidget::setFilterString);
+	connect(m_ui->addonsViewWidget, &ItemViewWidget::customContextMenuRequested, this, &AddonsContentsWidget::showContextMenu);
+	connect(m_ui->addonsViewWidget, &ItemViewWidget::clicked, this, &AddonsContentsWidget::save);
 }
 
 AddonsContentsWidget::~AddonsContentsWidget()
@@ -65,20 +64,59 @@ AddonsContentsWidget::~AddonsContentsWidget()
 
 void AddonsContentsWidget::changeEvent(QEvent *event)
 {
-	QWidget::changeEvent(event);
+	ContentsWidget::changeEvent(event);
 
 	if (event->type() == QEvent::LanguageChange)
 	{
 		m_ui->retranslateUi(this);
+
+		for (int i = 0; i < m_model->rowCount(); ++i)
+		{
+			const QModelIndex index(m_model->index(i, 0));
+
+			if (static_cast<Addon::AddonType>(index.data(TypeRole).toInt()) == Addon::UserScriptType)
+			{
+				m_model->setData(index, tr("User Scripts"), Qt::DisplayRole);
+			}
+		}
 	}
 }
 
+QVector<Addon*> AddonsContentsWidget::getSelectedAddons() const
+{
+	const QModelIndexList indexes(m_ui->addonsViewWidget->selectionModel()->selectedIndexes());
+	QVector<Addon*> addons;
+	addons.reserve(indexes.count());
+
+	for (int i = 0; i < indexes.count(); ++i)
+	{
+		if (indexes.at(i).isValid() && indexes.at(i).parent() != m_model->invisibleRootItem()->index())
+		{
+			Addon::AddonType type(static_cast<Addon::AddonType>(indexes.at(i).parent().data(TypeRole).toInt()));
+
+			if (type == Addon::UserScriptType)
+			{
+				UserScript *script(AddonsManager::getUserScript(indexes.at(i).data(NameRole).toString()));
+
+				if (script)
+				{
+					addons.append(script);
+				}
+			}
+		}
+	}
+
+	addons.squeeze();
+
+	return addons;
+}
+
 void AddonsContentsWidget::populateAddons()
 {
 	m_types.clear();
 	m_types[Addon::UserScriptType] = 0;
 
-	QStandardItem *userScriptsItem(new QStandardItem(ThemesManager::getIcon(QLatin1String("addon-user-script"), false), tr("User Scripts")));
+	QStandardItem *userScriptsItem(new QStandardItem(ThemesManager::createIcon(QLatin1String("addon-user-script"), false), tr("User Scripts")));
 	userScriptsItem->setData(Addon::UserScriptType, TypeRole);
 
 	m_model->appendRow(userScriptsItem);
@@ -95,12 +133,18 @@ void AddonsContentsWidget::populateAddons()
 
 	m_isLoading = false;
 
-	emit loadingStateChanged(WindowsManager::FinishedLoadingState);
+	emit loadingStateChanged(WebWidget::FinishedLoadingState);
+
+	connect(AddonsManager::getInstance(), &AddonsManager::userScriptModified, this, &AddonsContentsWidget::updateAddon);
+	connect(m_ui->addonsViewWidget->selectionModel(), &QItemSelectionModel::selectionChanged, [&]()
+	{
+		emit arbitraryActionsStateChanged({ActionsManager::DeleteAction});
+	});
 }
 
 void AddonsContentsWidget::addAddon()
 {
-	const QStringList sourcePaths(QFileDialog::getOpenFileNames(this, tr("Select Files"), QStandardPaths::standardLocations(QStandardPaths::HomeLocation).value(0), Utils::formatFileTypes(QStringList(tr("User Script files (*.js)")))));
+	const QStringList sourcePaths(QFileDialog::getOpenFileNames(this, tr("Select Files"), QStandardPaths::standardLocations(QStandardPaths::HomeLocation).value(0), Utils::formatFileTypes({tr("User Script files (*.js)")})));
 
 	if (sourcePaths.isEmpty())
 	{
@@ -190,7 +234,7 @@ void AddonsContentsWidget::addAddon()
 
 	if (!failedPaths.isEmpty())
 	{
-		QMessageBox::critical(this, tr("Error"), tr("Failed to import following User Script file(s):\n%1", "" , failedPaths.count()).arg(failedPaths.join(QLatin1Char('\n'))), QMessageBox::Close);
+		QMessageBox::critical(this, tr("Error"), tr("Failed to import following User Script file(s):\n%1", "", failedPaths.count()).arg(failedPaths.join(QLatin1Char('\n'))), QMessageBox::Close);
 	}
 
 	save();
@@ -217,7 +261,7 @@ void AddonsContentsWidget::addAddon(Addon *addon)
 		return;
 	}
 
-	QStandardItem *item(new QStandardItem(addon->getIcon(), (addon->getVersion().isEmpty() ? addon->getTitle() : QStringLiteral("%1 %2").arg(addon->getTitle()).arg(addon->getVersion()))));
+	QStandardItem *item(new QStandardItem(getAddonIcon(addon), (addon->getVersion().isEmpty() ? addon->getTitle() : QStringLiteral("%1 %2").arg(addon->getTitle()).arg(addon->getVersion()))));
 	item->setFlags(item->flags() | Qt::ItemNeverHasChildren);
 	item->setCheckable(true);
 	item->setCheckState(addon->isEnabled() ? Qt::Checked : Qt::Unchecked);
@@ -225,7 +269,7 @@ void AddonsContentsWidget::addAddon(Addon *addon)
 
 	if (addon->getType() == Addon::UserScriptType)
 	{
-		UserScript *script(qobject_cast<UserScript*>(addon));
+		const UserScript *script(static_cast<UserScript*>(addon));
 
 		if (script)
 		{
@@ -236,6 +280,52 @@ void AddonsContentsWidget::addAddon(Addon *addon)
 	typeItem->appendRow(item);
 }
 
+void AddonsContentsWidget::updateAddon(const QString &name)
+{
+	const QStandardItem *userScriptsItem(m_model->item(m_types.value(Addon::UserScriptType)));
+
+	if (!userScriptsItem)
+	{
+		return;
+	}
+
+	for (int i = 0; i < userScriptsItem->rowCount(); ++i)
+	{
+		const QModelIndex index(userScriptsItem->child(i)->index());
+
+		if (index.isValid() && index.data(NameRole).toString() == name)
+		{
+			UserScript *script(AddonsManager::getUserScript(name));
+
+			if (script)
+			{
+				m_ui->addonsViewWidget->setData(index, getAddonIcon(script), Qt::DecorationRole);
+				m_ui->addonsViewWidget->setData(index, (script->getVersion().isEmpty() ? script->getTitle() : QStringLiteral("%1 %2").arg(script->getTitle()).arg(script->getVersion())), Qt::DisplayRole);
+			}
+
+			break;
+		}
+	}
+}
+
+void AddonsContentsWidget::openAddon()
+{
+	const QVector<Addon*> addons(getSelectedAddons());
+
+	for (int i = 0; i < addons.count(); ++i)
+	{
+		if (addons.at(i)->getType() == Addon::UserScriptType)
+		{
+			const UserScript *script(static_cast<UserScript*>(addons.at(i)));
+
+			if (script)
+			{
+				Utils::runApplication({}, QUrl(script->getPath()));
+			}
+		}
+	}
+}
+
 void AddonsContentsWidget::reloadAddon()
 {
 	const QModelIndexList indexes(m_ui->addonsViewWidget->selectionModel()->selectedIndexes());
@@ -253,6 +343,9 @@ void AddonsContentsWidget::reloadAddon()
 				if (script)
 				{
 					script->reload();
+
+					m_ui->addonsViewWidget->setData(indexes.at(i), getAddonIcon(script), Qt::DecorationRole);
+					m_ui->addonsViewWidget->setData(indexes.at(i), (script->getVersion().isEmpty() ? script->getTitle() : QStringLiteral("%1 %2").arg(script->getTitle()).arg(script->getVersion())), Qt::DisplayRole);
 				}
 			}
 		}
@@ -261,74 +354,93 @@ void AddonsContentsWidget::reloadAddon()
 
 void AddonsContentsWidget::removeAddons()
 {
-	const QModelIndexList indexes(m_ui->addonsViewWidget->selectionModel()->selectedIndexes());
+	const QVector<Addon*> addons(getSelectedAddons());
 
-	if (indexes.isEmpty())
+	if (addons.isEmpty())
 	{
 		return;
 	}
 
-//TODO
+	QMessageBox messageBox;
+	messageBox.setWindowTitle(tr("Question"));
+	messageBox.setText(tr("You are about to irreversibly remove %n addon(s).", "", addons.count()));
+	messageBox.setInformativeText(tr("Do you want to continue?"));
+	messageBox.setIcon(QMessageBox::Question);
+	messageBox.setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
+	messageBox.setDefaultButton(QMessageBox::Yes);
+
+	if (messageBox.exec() == QMessageBox::Yes)
+	{
+		for (int i = 0; i < addons.count(); ++i)
+		{
+			if (addons.at(i)->canRemove())
+			{
+				addons.at(i)->remove();
+			}
+		}
+	}
+
+	AddonsManager::loadUserScripts();
+
+	save();
 }
 
 void AddonsContentsWidget::save()
 {
-	QStandardItem *userScriptsItem(nullptr);
-
-	if (m_types.contains(Addon::UserScriptType))
-	{
-		userScriptsItem = m_model->item(m_types[Addon::UserScriptType]);
-	}
+	const QStandardItem *userScriptsItem(m_model->item(m_types.value(Addon::UserScriptType)));
 
 	if (!userScriptsItem)
 	{
 		return;
 	}
 
-	QFile file(SessionsManager::getWritableDataPath(QLatin1String("scripts/scripts.json")));
-
-	if (!file.open(QIODevice::WriteOnly))
-	{
-		return;
-	}
-
-	QJsonObject settings;
+	QModelIndexList indexesToRemove;
+	QJsonObject settingsObject;
 
 	for (int i = 0; i < userScriptsItem->rowCount(); ++i)
 	{
-		QStandardItem *item(userScriptsItem->child(i));
+		const QModelIndex index(userScriptsItem->child(i)->index());
+		const QString name(index.data(NameRole).toString());
 
-		if (item && !item->data(NameRole).toString().isEmpty())
+		if (index.isValid())
 		{
-			QJsonObject script;
-			script.insert(QLatin1String("isEnabled"), QJsonValue(item->checkState() == Qt::Checked));
-
-			settings.insert(item->data(NameRole).toString(), script);
+			if (!name.isEmpty() && AddonsManager::getUserScript(name))
+			{
+				settingsObject.insert(name, QJsonObject({{QLatin1String("isEnabled"), QJsonValue(index.data(Qt::CheckStateRole).toInt() == Qt::Checked)}}));
+			}
+			else
+			{
+				indexesToRemove.append(index);
+			}
 		}
 	}
 
-	QJsonDocument document;
-	document.setObject(settings);
+	JsonSettings settings;
+	settings.setObject(settingsObject);
+	settings.save(SessionsManager::getWritableDataPath(QLatin1String("scripts/scripts.json")));
 
-	file.write(document.toJson(QJsonDocument::Indented));
-	file.close();
+	for (int i = (indexesToRemove.count() - 1); i >= 0; --i)
+	{
+		m_ui->addonsViewWidget->model()->removeRow(indexesToRemove.at(i).row(), indexesToRemove.at(i).parent());
+	}
 }
 
-void AddonsContentsWidget::showContextMenu(const QPoint &point)
+void AddonsContentsWidget::showContextMenu(const QPoint &position)
 {
-	const QModelIndex index(m_ui->addonsViewWidget->indexAt(point));
+	const QVector<Addon*> addons(getSelectedAddons());
 	QMenu menu(this);
+	menu.addAction(tr("Add Addon…"), this, static_cast<void(AddonsContentsWidget::*)()>(&AddonsContentsWidget::addAddon));
 
-	if (index.isValid() && index.parent() != m_model->invisibleRootItem()->index())
+	if (!addons.isEmpty())
 	{
-		menu.addAction(tr("Reload Addon"), this, SLOT(reloadAddon()));
 		menu.addSeparator();
-		menu.addAction(tr("Remove Addon…"), this, SLOT(removeAddons()))->setEnabled(false);
+		menu.addAction(tr("Open Addon File"), this, &AddonsContentsWidget::openAddon);
+		menu.addAction(tr("Reload Addon"), this, &AddonsContentsWidget::reloadAddon);
 		menu.addSeparator();
+		menu.addAction(tr("Remove Addon…"), this, &AddonsContentsWidget::removeAddons);
 	}
 
-	menu.addAction(tr("Add Addon…"), this, SLOT(addAddon()));
-	menu.exec(m_ui->addonsViewWidget->mapToGlobal(point));
+	menu.exec(m_ui->addonsViewWidget->mapToGlobal(position));
 }
 
 void AddonsContentsWidget::print(QPrinter *printer)
@@ -336,10 +448,8 @@ void AddonsContentsWidget::print(QPrinter *printer)
 	m_ui->addonsViewWidget->render(printer);
 }
 
-void AddonsContentsWidget::triggerAction(int identifier, const QVariantMap &parameters)
+void AddonsContentsWidget::triggerAction(int identifier, const QVariantMap &parameters, ActionsManager::TriggerType trigger)
 {
-	Q_UNUSED(parameters)
-
 	switch (identifier)
 	{
 		case ActionsManager::SelectAllAction:
@@ -352,8 +462,7 @@ void AddonsContentsWidget::triggerAction(int identifier, const QVariantMap &para
 			break;
 		case ActionsManager::FindAction:
 		case ActionsManager::QuickFindAction:
-		case ActionsManager::ActivateAddressFieldAction:
-			m_ui->filterLineEdit->setFocus();
+			m_ui->filterLineEditWidget->setFocus();
 
 			break;
 		case ActionsManager::ActivateContentAction:
@@ -361,42 +470,15 @@ void AddonsContentsWidget::triggerAction(int identifier, const QVariantMap &para
 
 			break;
 		default:
+			ContentsWidget::triggerAction(identifier, parameters, trigger);
+
 			break;
 	}
 }
 
-void AddonsContentsWidget::filterAddons(const QString &filter)
-{
-	for (int i = 0; i < m_model->rowCount(); ++i)
-	{
-		m_ui->addonsViewWidget->setRowHidden(i, m_model->invisibleRootItem()->index(), (!filter.isEmpty() && !m_model->item(i, 0)->data(Qt::DisplayRole).toString().contains(filter, Qt::CaseInsensitive)));
-	}
-}
-
-Action* AddonsContentsWidget::getAction(int identifier)
-{
-	if (m_actions.contains(identifier))
-	{
-		return m_actions[identifier];
-	}
-
-	if (identifier != ActionsManager::DeleteAction && identifier != ActionsManager::SelectAllAction)
-	{
-		return nullptr;
-	}
-
-	Action *action(new Action(identifier, this));
-
-	m_actions[identifier] = action;
-
-	connect(action, SIGNAL(triggered()), this, SLOT(triggerAction()));
-
-	return action;
-}
-
 QString AddonsContentsWidget::getTitle() const
 {
-	return tr("Addons Manager");
+	return tr("Addons");
 }
 
 QLatin1String AddonsContentsWidget::getType() const
@@ -411,35 +493,47 @@ QUrl AddonsContentsWidget::getUrl() const
 
 QIcon AddonsContentsWidget::getIcon() const
 {
-	return ThemesManager::getIcon(QLatin1String("preferences-plugin"), false);
+	return ThemesManager::createIcon(QLatin1String("preferences-plugin"), false);
 }
 
-WindowsManager::LoadingState AddonsContentsWidget::getLoadingState() const
+QIcon AddonsContentsWidget::getAddonIcon(Addon *addon) const
 {
-	return (m_isLoading ? WindowsManager::OngoingLoadingState : WindowsManager::FinishedLoadingState);
+	return ((!addon || addon->getIcon().isNull()) ? ThemesManager::createIcon(QLatin1String("addon-user-script"), false) : addon->getIcon());
+}
+
+ActionsManager::ActionDefinition::State AddonsContentsWidget::getActionState(int identifier, const QVariantMap &parameters) const
+{
+	ActionsManager::ActionDefinition::State state(ActionsManager::getActionDefinition(identifier).getDefaultState());
+
+	switch (identifier)
+	{
+		case ActionsManager::SelectAllAction:
+			state.isEnabled = true;
+
+			return state;
+		case ActionsManager::DeleteAction:
+			state.isEnabled = (m_ui->addonsViewWidget->selectionModel() && !m_ui->addonsViewWidget->selectionModel()->selectedIndexes().isEmpty());
+
+			return state;
+		default:
+			break;
+	}
+
+	return ContentsWidget::getActionState(identifier, parameters);
+}
+
+WebWidget::LoadingState AddonsContentsWidget::getLoadingState() const
+{
+	return (m_isLoading ? WebWidget::OngoingLoadingState : WebWidget::FinishedLoadingState);
 }
 
 bool AddonsContentsWidget::eventFilter(QObject *object, QEvent *event)
 {
-	if (object == m_ui->addonsViewWidget && event->type() == QEvent::KeyPress)
+	if (object == m_ui->addonsViewWidget && event->type() == QEvent::KeyPress && static_cast<QKeyEvent*>(event)->key() == Qt::Key_Delete)
 	{
-		QKeyEvent *keyEvent(static_cast<QKeyEvent*>(event));
+		removeAddons();
 
-		if (keyEvent && keyEvent->key() == Qt::Key_Delete)
-		{
-			removeAddons();
-
-			return true;
-		}
-	}
-	else if (object == m_ui->filterLineEdit && event->type() == QEvent::KeyPress)
-	{
-		QKeyEvent *keyEvent(static_cast<QKeyEvent*>(event));
-
-		if (keyEvent->key() == Qt::Key_Escape)
-		{
-			m_ui->filterLineEdit->clear();
-		}
+		return true;
 	}
 
 	return ContentsWidget::eventFilter(object, event);
