@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -35,42 +35,46 @@ namespace Ui
 
 class Window;
 
-class BookmarksContentsWidget : public ContentsWidget
+class BookmarksContentsWidget final : public ContentsWidget
 {
 	Q_OBJECT
 
 public:
-	explicit BookmarksContentsWidget(Window *window);
+	explicit BookmarksContentsWidget(const QVariantMap &parameters, Window *window, QWidget *parent);
 	~BookmarksContentsWidget();
 
-	void print(QPrinter *printer);
-	Action* getAction(int identifier);
-	QString getTitle() const;
-	QLatin1String getType() const;
-	QUrl getUrl() const;
-	QIcon getIcon() const;
-	bool eventFilter(QObject *object, QEvent *event);
+	void print(QPrinter *printer) override;
+	QString getTitle() const override;
+	QLatin1String getType() const override;
+	QUrl getUrl() const override;
+	QIcon getIcon() const override;
+	ActionsManager::ActionDefinition::State getActionState(int identifier, const QVariantMap &parameters = {}) const override;
+	bool eventFilter(QObject *object, QEvent *event) override;
 
 public slots:
-	void triggerAction(int identifier, const QVariantMap &parameters = QVariantMap());
+	void triggerAction(int identifier, const QVariantMap &parameters = {}, ActionsManager::TriggerType trigger = ActionsManager::UnknownTrigger) override;
 
 protected:
-	void changeEvent(QEvent *event);
-	BookmarksItem* findFolder(const QModelIndex &index);
+	struct BookmarkLocation final
+	{
+		BookmarksModel::Bookmark *folder = nullptr;
+		int row = -1;
+	};
+
+	void changeEvent(QEvent *event) override;
+	BookmarkLocation getBookmarkCreationLocation();
 
 protected slots:
 	void addBookmark();
 	void addFolder();
 	void addSeparator();
 	void removeBookmark();
-	void restoreBookmark();
-	void openBookmark(const QModelIndex &index = QModelIndex());
+	void openBookmark();
 	void bookmarkProperties();
-	void showContextMenu(const QPoint &point);
+	void showContextMenu(const QPoint &position);
 	void updateActions();
 
 private:
-	QHash<int, Action*> m_actions;
 	Ui::BookmarksContentsWidget *m_ui;
 };
 
