@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -34,47 +34,45 @@ namespace Ui
 
 class Window;
 
-class CacheContentsWidget : public ContentsWidget
+class CacheContentsWidget final : public ContentsWidget
 {
 	Q_OBJECT
 
 public:
-	explicit CacheContentsWidget(Window *window);
+	explicit CacheContentsWidget(const QVariantMap &parameters, Window *window, QWidget *parent);
 	~CacheContentsWidget();
 
-	void print(QPrinter *printer);
-	Action* getAction(int identifier);
-	QString getTitle() const;
-	QLatin1String getType() const;
-	QUrl getUrl() const;
-	QIcon getIcon() const;
-	WindowsManager::LoadingState getLoadingState() const;
-	bool eventFilter(QObject *object, QEvent *event);
+	void print(QPrinter *printer) override;
+	QString getTitle() const override;
+	QLatin1String getType() const override;
+	QUrl getUrl() const override;
+	QIcon getIcon() const override;
+	ActionsManager::ActionDefinition::State getActionState(int identifier, const QVariantMap &parameters = {}) const override;
+	WebWidget::LoadingState getLoadingState() const override;
+	bool eventFilter(QObject *object, QEvent *event) override;
 
 public slots:
-	void triggerAction(int identifier, const QVariantMap &parameters = QVariantMap());
+	void triggerAction(int identifier, const QVariantMap &parameters = {}, ActionsManager::TriggerType trigger = ActionsManager::UnknownTrigger) override;
 
 protected:
-	void changeEvent(QEvent *event);
+	void changeEvent(QEvent *event) override;
 	QStandardItem* findDomain(const QString &domain);
-	QStandardItem* findEntry(const QUrl &entry);
 	QUrl getEntry(const QModelIndex &index) const;
 
 protected slots:
 	void populateCache();
-	void addEntry(const QUrl &entry);
-	void removeEntry(const QUrl &entry);
 	void removeEntry();
 	void removeDomainEntries();
 	void removeDomainEntriesOrEntry();
-	void openEntry(const QModelIndex &index = QModelIndex());
+	void openEntry();
 	void copyEntryLink();
-	void showContextMenu(const QPoint &point);
+	void handleEntryAdded(const QUrl &entry);
+	void handleEntryRemoved(const QUrl &entry);
+	void showContextMenu(const QPoint &position);
 	void updateActions();
 
 private:
 	QStandardItemModel *m_model;
-	QHash<int, Action*> m_actions;
 	bool m_isLoading;
 	Ui::CacheContentsWidget *m_ui;
 };
