@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -35,41 +35,48 @@ namespace Ui
 
 class Window;
 
-class HistoryContentsWidget : public ContentsWidget
+class HistoryContentsWidget final : public ContentsWidget
 {
 	Q_OBJECT
 
 public:
-	explicit HistoryContentsWidget(Window *window);
+	enum DataRole
+	{
+		IdentifierRole = Qt::UserRole,
+		TimeVisitedRole,
+		GroupDateRole
+	};
+
+	explicit HistoryContentsWidget(const QVariantMap &parameters, Window *window, QWidget *parent);
 	~HistoryContentsWidget();
 
-	void print(QPrinter *printer);
-	QString getTitle() const;
-	QLatin1String getType() const;
-	QUrl getUrl() const;
-	QIcon getIcon() const;
-	WindowsManager::LoadingState getLoadingState() const;
-	bool eventFilter(QObject *object, QEvent *event);
+	void print(QPrinter *printer) override;
+	QString getTitle() const override;
+	QLatin1String getType() const override;
+	QUrl getUrl() const override;
+	QIcon getIcon() const override;
+	WebWidget::LoadingState getLoadingState() const override;
+	bool eventFilter(QObject *object, QEvent *event) override;
 
 public slots:
-	void triggerAction(int identifier, const QVariantMap &parameters = QVariantMap());
+	void triggerAction(int identifier, const QVariantMap &parameters = {}, ActionsManager::TriggerType trigger = ActionsManager::UnknownTrigger) override;
 
 protected:
-	void changeEvent(QEvent *event);
+	void changeEvent(QEvent *event) override;
 	QStandardItem* findEntry(quint64 identifier);
 	quint64 getEntry(const QModelIndex &index) const;
 
 protected slots:
 	void populateEntries();
-	void addEntry(HistoryEntryItem *entry);
-	void modifyEntry(HistoryEntryItem *entry);
-	void removeEntry(HistoryEntryItem *entry);
 	void removeEntry();
 	void removeDomainEntries();
-	void openEntry(const QModelIndex &index = QModelIndex());
+	void openEntry();
 	void bookmarkEntry();
 	void copyEntryLink();
-	void showContextMenu(const QPoint &point);
+	void handleEntryAdded(HistoryModel::Entry *entry);
+	void handleEntryModified(HistoryModel::Entry *entry);
+	void handleEntryRemoved(HistoryModel::Entry *entry);
+	void showContextMenu(const QPoint &position);
 
 private:
 	QStandardItemModel *m_model;
