@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
 * Copyright (C) 2015 Piotr Wójcik <chocimier@tlen.pl>
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2017 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -10,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -27,7 +27,7 @@
 namespace Otter
 {
 
-ProxyModel::ProxyModel(QStandardItemModel *model, const QList<QPair<QString, int> > &mapping, QObject *parent) : QIdentityProxyModel(parent),
+ProxyModel::ProxyModel(QStandardItemModel *model, const QVector<QPair<QString, int> > &mapping, QObject *parent) : QIdentityProxyModel(parent),
 	m_model(model),
 	m_mapping(mapping)
 {
@@ -67,7 +67,7 @@ QVariant ProxyModel::data(const QModelIndex &index, int role) const
 
 	if (role == Qt::DecorationRole && index.column() > 0)
 	{
-		return QVariant();
+		return {};
 	}
 
 	return QIdentityProxyModel::data(index.sibling(index.row(), 0), role);
@@ -75,7 +75,20 @@ QVariant ProxyModel::data(const QModelIndex &index, int role) const
 
 QVariant ProxyModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
-	return (orientation == Qt::Horizontal && role == Qt::DisplayRole && section >= 0 && section < m_mapping.count()) ? QCoreApplication::translate("views", m_mapping[section].first.toUtf8().constData()) : QIdentityProxyModel::headerData(section, orientation, role);
+	if (orientation == Qt::Horizontal && section >= 0 && section < m_mapping.count())
+	{
+		if (role == Qt::DisplayRole)
+		{
+			return QCoreApplication::translate("views", m_mapping[section].first.toUtf8().constData());
+		}
+
+		if (m_headerData.contains(section) && m_headerData[section].contains(role))
+		{
+			return m_headerData[section][role];
+		}
+	}
+
+	return QIdentityProxyModel::headerData(section, orientation, role);
 }
 
 QModelIndex ProxyModel::index(int row, int column, const QModelIndex &parent) const
@@ -113,4 +126,23 @@ int ProxyModel::rowCount(const QModelIndex &parent) const
 	return (m_model ? m_model->rowCount(mapToSource(parent)) : 0);
 }
 
+bool ProxyModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role)
+{
+	if (orientation == Qt::Horizontal)
+	{
+		if (!m_headerData.contains(section))
+		{
+			m_headerData[section] = {};
+		}
+
+		m_headerData[section][role] = value;
+
+		emit headerDataChanged(orientation, section, section);
+
+		return true;
+	}
+
+	return false;
+}
+
 }
