@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,17 +9,18 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "SaveSessionDialog.h"
 #include "MainWindow.h"
 #include "../core/SessionsManager.h"
+#include "../core/Utils.h"
 
 #include "ui_SaveSessionDialog.h"
 
@@ -32,13 +33,20 @@ namespace Otter
 SaveSessionDialog::SaveSessionDialog(QWidget *parent) : Dialog(parent),
 	m_ui(new Ui::SaveSessionDialog)
 {
-	m_ui->setupUi(this);
-	m_ui->titleLineEdit->setText(SessionsManager::getSession(SessionsManager::getCurrentSession()).title);
-	m_ui->identifierLineEdit->setText(SessionsManager::getCurrentSession());
-	m_ui->identifierLineEdit->setValidator(new QRegularExpressionValidator(QRegularExpression(QLatin1String("[a-z0-9\\-_]+")), this));
+	QString identifier(SessionsManager::getCurrentSession());
 
-	connect(m_ui->buttonBox, SIGNAL(accepted()), this, SLOT(saveSession()));
-	connect(m_ui->buttonBox, SIGNAL(rejected()), this, SLOT(close()));
+	if (identifier == QLatin1String("default"))
+	{
+		identifier = Utils::createIdentifier(QLatin1String("default"), SessionsManager::getSessions());
+	}
+
+	m_ui->setupUi(this);
+	m_ui->titleLineEditWidget->setText(SessionsManager::getSession(SessionsManager::getCurrentSession()).title);
+	m_ui->identifierLineEditWidget->setText(identifier);
+	m_ui->identifierLineEditWidget->setValidator(new QRegularExpressionValidator(QRegularExpression(QLatin1String("[a-z0-9\\-_]+")), this));
+
+	connect(m_ui->buttonBox, &QDialogButtonBox::accepted, this, &SaveSessionDialog::saveSession);
+	connect(m_ui->buttonBox, &QDialogButtonBox::rejected, this, &SaveSessionDialog::close);
 }
 
 SaveSessionDialog::~SaveSessionDialog()
@@ -58,21 +66,16 @@ void SaveSessionDialog::changeEvent(QEvent *event)
 
 void SaveSessionDialog::saveSession()
 {
-	if (m_ui->identifierLineEdit->text().isEmpty())
+	const QString identifier(m_ui->identifierLineEditWidget->text());
+
+	if (identifier.isEmpty() || (SessionsManager::getCurrentSession() != identifier && SessionsManager::getSession(identifier).isValid() && QMessageBox::question(this, tr("Question"), tr("Session with specified indentifier already exists.\nDo you want to overwrite it?"), QMessageBox::Yes, QMessageBox::No) == QMessageBox::No))
 	{
 		show();
 
 		return;
 	}
 
-	if (m_ui->identifierLineEdit->text() != SessionsManager::getCurrentSession() && SessionsManager::getSession(m_ui->identifierLineEdit->text()).windows.count() > 0 && QMessageBox::question(this, tr("Question"), tr("Session with specified indentifier already exists.\nDo you want to overwrite it?"), QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
-	{
-		show();
-
-		return;
-	}
-
-	if (SessionsManager::saveSession(m_ui->identifierLineEdit->text(), m_ui->titleLineEdit->text(), (m_ui->onlyCurrentWindowCheckBox->isChecked() ? qobject_cast<MainWindow*>(parentWidget()) : NULL)))
+	if (SessionsManager::saveSession(identifier, m_ui->titleLineEditWidget->text(), (m_ui->onlyCurrentWindowCheckBox->isChecked() ? qobject_cast<MainWindow*>(parentWidget()) : nullptr)))
 	{
 		close();
 	}
