@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,17 +9,17 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "TransferDialog.h"
 #include "../core/HandlersManager.h"
-#include "../core/Transfer.h"
+#include "../core/ThemesManager.h"
 #include "../core/TransfersManager.h"
 #include "../core/Utils.h"
 
@@ -27,6 +27,7 @@
 
 #include <QtCore/QFileInfo>
 #include <QtCore/QtMath>
+#include <QtWidgets/QFileIconProvider>
 
 namespace Otter
 {
@@ -35,8 +36,9 @@ TransferDialog::TransferDialog(Transfer *transfer, QWidget *parent) : Dialog(par
 	m_transfer(transfer),
 	m_ui(new Ui::TransferDialog)
 {
-	const QPixmap icon = Utils::getIcon(transfer->getMimeType().iconName()).pixmap(16, 16);
-	QString fileName = transfer->getSuggestedFileName();
+	const QString iconName(transfer->getMimeType().iconName());
+	const QPixmap icon(QIcon::fromTheme(iconName, QFileIconProvider().icon(iconName)).pixmap(16, 16));
+	QString fileName(transfer->getSuggestedFileName());
 
 	if (fileName.isEmpty())
 	{
@@ -56,14 +58,14 @@ TransferDialog::TransferDialog(Transfer *transfer, QWidget *parent) : Dialog(par
 
 	m_ui->nameTextLabelWidget->setText(fileName);
 	m_ui->typeTextLabelWidget->setText(transfer->getMimeType().comment());
-	m_ui->fromTextLabelWidget->setText(transfer->getSource().host().isEmpty() ? QLatin1String("localhost") : transfer->getSource().host());
+	m_ui->fromTextLabelWidget->setText(Utils::extractHost(transfer->getSource()));
 	m_ui->openWithComboBoxWidget->setMimeType(transfer->getMimeType());
 
 	setProgress(m_transfer->getBytesReceived(), m_transfer->getBytesTotal());
 	setWindowTitle(tr("Opening %1").arg(fileName));
 
-	connect(transfer, SIGNAL(progressChanged(qint64,qint64)), this, SLOT(setProgress(qint64,qint64)));
-	connect(m_ui->buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(buttonClicked(QAbstractButton*)));
+	connect(transfer, &Transfer::progressChanged, this, &TransferDialog::setProgress);
+	connect(m_ui->buttonBox, &QDialogButtonBox::clicked, this, &TransferDialog::handleButtonClicked);
 }
 
 TransferDialog::~TransferDialog()
@@ -75,15 +77,15 @@ void TransferDialog::changeEvent(QEvent *event)
 {
 	QWidget::changeEvent(event);
 
-	if (event->type() ==  QEvent::LanguageChange)
+	if (event->type() == QEvent::LanguageChange)
 	{
 		m_ui->retranslateUi(this);
 	}
 }
 
-void TransferDialog::buttonClicked(QAbstractButton *button)
+void TransferDialog::handleButtonClicked(QAbstractButton *button)
 {
-	const QDialogButtonBox::StandardButton standardButton = m_ui->buttonBox->standardButton(button);
+	const QDialogButtonBox::StandardButton standardButton(m_ui->buttonBox->standardButton(button));
 
 	if (!m_transfer || (standardButton != QDialogButtonBox::Open && standardButton != QDialogButtonBox::Save))
 	{
@@ -93,10 +95,10 @@ void TransferDialog::buttonClicked(QAbstractButton *button)
 
 			if (m_ui->rememberChoiceCheckBox->isChecked())
 			{
-				HandlerDefinition definition;
-				definition.transferMode = IgnoreTransferMode;
+				HandlersManager::HandlerDefinition definition;
+				definition.transferMode = HandlersManager::HandlerDefinition::IgnoreTransfer;
 
-				HandlersManager::setHandler(m_transfer->getMimeType().name(), definition);
+				HandlersManager::setHandler(m_transfer->getMimeType(), definition);
 			}
 		}
 
@@ -111,16 +113,16 @@ void TransferDialog::buttonClicked(QAbstractButton *button)
 
 		if (m_ui->rememberChoiceCheckBox->isChecked())
 		{
-			HandlerDefinition definition;
-			definition.transferMode = OpenTransferMode;
+			HandlersManager::HandlerDefinition definition;
+			definition.transferMode = HandlersManager::HandlerDefinition::OpenTransfer;
 			definition.openCommand = m_ui->openWithComboBoxWidget->getCommand();
 
-			HandlersManager::setHandler(m_transfer->getMimeType().name(), definition);
+			HandlersManager::setHandler(m_transfer->getMimeType(), definition);
 		}
 	}
 	else if (standardButton == QDialogButtonBox::Save)
 	{
-		QWidget *dialog = parentWidget();
+		QWidget *dialog(parentWidget());
 
 		while (dialog)
 		{
@@ -134,7 +136,7 @@ void TransferDialog::buttonClicked(QAbstractButton *button)
 			dialog = dialog->parentWidget();
 		}
 
-		const QString path = TransfersManager::getSavePath(m_transfer->getSuggestedFileName());
+		const QString path(Utils::getSavePath(m_transfer->getSuggestedFileName()).path);
 
 		if (path.isEmpty())
 		{
@@ -145,15 +147,15 @@ void TransferDialog::buttonClicked(QAbstractButton *button)
 			return;
 		}
 
-		m_transfer->setTarget(path);
+		m_transfer->setTarget(path, true);
 
 		if (m_ui->rememberChoiceCheckBox->isChecked())
 		{
-			HandlerDefinition definition;
-			definition.transferMode = SaveTransferMode;
+			HandlersManager::HandlerDefinition definition;
+			definition.transferMode = HandlersManager::HandlerDefinition::SaveTransfer;
 			definition.downloadsPath = QFileInfo(path).canonicalPath();
 
-			HandlersManager::setHandler(m_transfer->getMimeType().name(), definition);
+			HandlersManager::setHandler(m_transfer->getMimeType(), definition);
 		}
 	}
 
@@ -174,9 +176,7 @@ void TransferDialog::setProgress(qint64 bytesReceived, qint64 bytesTotal)
 	}
 	else
 	{
-		const int progress = ((bytesReceived > 0 && bytesTotal > 0) ? qFloor((static_cast<qreal>(bytesReceived) / bytesTotal) * 100) : 0);
-
-		m_ui->sizeTextLabelWidget->setText(tr("%1 (%2% downloaded)").arg(Utils::formatUnit(bytesTotal)).arg(progress));
+		m_ui->sizeTextLabelWidget->setText(tr("%1 (%2% downloaded)").arg(Utils::formatUnit(bytesTotal)).arg(Utils::calculatePercent(bytesReceived, bytesTotal), 0, 'f', 1));
 	}
 }
 
