@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
-* Copyright (C) 2014 Piotr Wójcik <chocimier@tlen.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2014 - 2016 Piotr Wójcik <chocimier@tlen.pl>
 * Copyright (C) 2015 - 2016 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
@@ -11,11 +11,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -35,24 +35,35 @@ namespace Ui
 	class WebsitePreferencesDialog;
 }
 
-class WebsitePreferencesDialog : public Dialog
+class WebsitePreferencesDialog final : public Dialog
 {
 	Q_OBJECT
 
 public:
-	explicit WebsitePreferencesDialog(const QUrl &url, const QList<QNetworkCookie> &cookies, QWidget *parent = nullptr);
+	explicit WebsitePreferencesDialog(const QString &host, const QVector<QNetworkCookie> &cookies, QWidget *parent = nullptr);
 	~WebsitePreferencesDialog();
 
+	QVector<QNetworkCookie> getCookiesToDelete() const;
+	QVector<QNetworkCookie> getCookiesToInsert() const;
+	QString getHost() const;
+
 protected:
-	void changeEvent(QEvent *event);
+	void changeEvent(QEvent *event) override;
 
 protected slots:
-	void buttonClicked(QAbstractButton *button);
+	void addCookie(const QNetworkCookie &cookie);
+	void addNewCookie();
+	void removeCookie();
+	void cookieProperties();
+	void handleButtonClicked(QAbstractButton *button);
+	void handleValueChanged();
+	void updateCookiesActions();
 	void updateContentBlockingProfile(const QString &name);
-	void updateValues(bool checked = false);
-	void valueChanged();
+	void updateValues(bool isChecked = false);
 
 private:
+	QVector<QNetworkCookie> m_cookiesToDelete;
+	QVector<QNetworkCookie> m_cookiesToInsert;
 	bool m_updateOverride;
 	Ui::WebsitePreferencesDialog *m_ui;
 };
