@@ -1,6 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
 * Copyright (C) 2016 Jan Bajer aka bajasoft <jbajer@gmail.com>
+* Copyright (C) 2016 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -30,26 +31,26 @@ namespace Ui
 	class ContentBlockingProfileDialog;
 }
 
-class ContentBlockingProfile;
+class ContentFiltersProfile;
 
-class ContentBlockingProfileDialog : public Dialog
+class ContentBlockingProfileDialog final : public Dialog
 {
 	Q_OBJECT
 
 public:
-	explicit ContentBlockingProfileDialog(QWidget *parent = nullptr, ContentBlockingProfile *profile = nullptr);
+	explicit ContentBlockingProfileDialog(QWidget *parent = nullptr, ContentFiltersProfile *profile = nullptr);
 	~ContentBlockingProfileDialog();
 
-	ContentBlockingProfile* getProfile();
+	ContentFiltersProfile* getProfile();
 
 protected:
-	void changeEvent(QEvent *event);
+	void changeEvent(QEvent *event) override;
 
 protected slots:
 	void save();
 
 private:
-	ContentBlockingProfile *m_profile;
+	ContentFiltersProfile *m_profile;
 	Ui::ContentBlockingProfileDialog *m_ui;
 };
 
