@@ -1,6 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2017 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -22,6 +23,7 @@
 
 #include "../Dialog.h"
 #include "../ItemDelegate.h"
+#include "../../core/GesturesManager.h"
 
 namespace Otter
 {
@@ -31,52 +33,47 @@ namespace Ui
 	class MouseProfileDialog;
 }
 
-struct MouseProfile
-{
-	QString identifier;
-	QString title;
-	QString description;
-	QString author;
-	QString version;
-	QHash<QString, QHash<QString, int> > gestures;
-	bool isModified = false;
-};
-
-class GestureActionDelegate : public ItemDelegate
+class GestureActionDelegate final : public ItemDelegate
 {
 public:
 	explicit GestureActionDelegate(QObject *parent);
 
-	void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
-	void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
-	QWidget* createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
+	void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const override;
+	QWidget* createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
 };
 
-class MouseProfileDialog : public Dialog
+class MouseProfileDialog final : public Dialog
 {
 	Q_OBJECT
 
 public:
+	enum DataRole
+	{
+		IdentifierRole = Qt::UserRole,
+		ContextRole,
+		NameRole,
+		ParametersRole
+	};
+
 	explicit MouseProfileDialog(const QString &profile, const QHash<QString, MouseProfile> &profiles, QWidget *parent = nullptr);
 	~MouseProfileDialog();
 
 	MouseProfile getProfile() const;
+	bool isModified() const;
 
 protected:
-	void changeEvent(QEvent *event);
+	void changeEvent(QEvent *event) override;
 
 protected slots:
 	void addGesture();
 	void removeGesture();
-	void saveGesture();
 	void addStep();
 	void removeStep();
 	void updateGesturesActions();
 	void updateStepsActions();
 
 private:
-	QString m_profile;
-	bool m_isModified;
+	MouseProfile m_profile;
 	Ui::MouseProfileDialog *m_ui;
 };
 
